﻿using System.Drawing;

namespace TestProject
{
    /// <summary>
    /// 선
    /// </summary>
    public class Line
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 명칭
        /// </summary>
        public string Name = string.Empty;
            
        /// <summary>
        /// 표시 여부
        /// </summary>
        public bool Visible = true;

        /// <summary>
        /// X 배열
        /// </summary>
        /// <remarks>NULL인 경우 Y 인덱스를 사용한다.</remarks>
        public double[] XArray = null;

        /// <summary>
        /// Y 배열
        /// </summary>
        public double[] YArray = null;

        /// <summary>
        /// 선 색상
        /// </summary>
        public Color LineColor = Color.Black;

        /// <summary>
        /// 선 너비
        /// </summary>
        public float LineWidth = 1f;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 최소 X - MinimumX

        /// <summary>
        /// 최소 X
        /// </summary>
        public double MinimumX
        {
            get
            {
                double [] xArray = XArray;

                if(xArray == null)
                {
                    return 0;
                }

                if(xArray.Length < 1)
                {
                    return 0;
                }

                double minimumX = xArray[0];

                for(int i = 0; i < xArray.Length; i++)
                {
                    minimumX = minimumX < xArray[i] ? minimumX : xArray[i];
                }

                return minimumX;
            }
        }

        #endregion
        #region 최대 X - MaximumX

        /// <summary>
        /// 최대 X
        /// </summary>
        public double MaximumX
        {
            get
            {
                double[] xArray = XArray;

                if(xArray == null)
                {
                    double[] yArray = YArray;

                    if(yArray == null)
                    {
                        return 0;
                    }

                    return yArray.Length;
                }

                if(xArray.Length < 1)
                {
                    return 0;
                }

                double maximumX = xArray[0];

                for(int i = 0; i < xArray.Length; i++)
                {
                    maximumX = maximumX > xArray[i] ? maximumX : xArray[i];
                }

                return maximumX;
            }
        }

        #endregion

        #region 최소 X - MinimumY

        /// <summary>
        /// 최소 X
        /// </summary>
        public double MinimumY
        {
            get
            {
                double[] yArray = YArray;

                if(yArray == null)
                {
                    return 0;
                }

                if(yArray.Length < 1)
                {
                    return 0;
                }

                double minimumY = yArray[0];

                for(int i = 0; i < yArray.Length; i++)
                {
                    minimumY = minimumY < yArray[i] ? minimumY : yArray[i];
                }

                return minimumY;
            }
        }

        #endregion
        #region 최대 Y - MaximumY

        /// <summary>
        /// 최대 Y
        /// </summary>
        public double MaximumY
        {
            get
            {
                double[] yArray = YArray;

                if(yArray == null)
                {
                    return 0;
                }

                if(yArray.Length < 1)
                {
                    return 0;
                }

                double maximumY = yArray[0];

                for(int i = 0; i < yArray.Length; i++)
                {
                    maximumY = maximumY > yArray[i] ? maximumY : yArray[i];
                }

                return maximumY;                
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 업데이트하기 - Update(xArray, yArray)

        /// <summary>
        /// 업데이트하기
        /// </summary>
        /// <param name="xArray">X 배열</param>
        /// <param name="yArray">Y 배열</param>
        public void Update(float[] xArray, float[] yArray)
        {
            if(xArray != null)
            {
                bool resize = XArray == null ? true : XArray.Length == xArray.Length ? false : true;

                double[] newXArray = resize ? new double[xArray.Length] : XArray;

                for(int i = 0; i < xArray.Length; i++)
                {
                    newXArray[i] = xArray[i];
                }

                XArray = newXArray;
            }
            else
            {
                XArray = null;
            }

            if(yArray != null)
            {
                bool resize = YArray == null ? true : YArray.Length == yArray.Length ? false : true;

                double[] newYArray = resize ? new double[yArray.Length] : YArray;

                for(int i = 0; i < yArray.Length; i++)
                {
                    newYArray[i] = yArray[i];
                }

                YArray = newYArray;
            }
            else
            {
                YArray = null;
            }                
        }

        #endregion
        #region 업데이트하기 - Update(yArray)

        /// <summary>
        /// 업데이트하기
        /// </summary>
        /// <param name="yArray">Y 배열</param>
        public void Update(float[] yArray)
        {
            Update(null, yArray);
        }

        #endregion

        #region 업데이트하기 - Update(xArray, yArray)

        /// <summary>
        /// 업데이트하기
        /// </summary>
        /// <param name="xArray">X 배열</param>
        /// <param name="yArray">Y 배열</param>
        public void Update(double[] xArray, double[] yArray)
        {
            if(xArray != null)
            {
                bool resize = XArray == null ? true : XArray.Length == xArray.Length ? false : true;

                double[] newXArray = resize ? new double[xArray.Length] : XArray;

                for(int i = 0; i < xArray.Length; i++)
                {
                    newXArray[i] = xArray[i];
                }

                XArray = newXArray;
            }
            else
            {
                XArray = null;
            }

            if(yArray != null)
            {
                bool resize = YArray == null ? true : YArray.Length == yArray.Length ? false : true;

                double[] newYArray = resize ? new double[yArray.Length] : YArray;

                for(int i = 0; i < yArray.Length; i++)
                {
                    newYArray[i] = yArray[i];
                }

                YArray = newYArray;
            }
            else
            {
                YArray = null;
            }
        }

        #endregion
        #region 업데이트하기 - Update(yArray)

        /// <summary>
        /// 업데이트하기
        /// </summary>
        /// <param name="yArray">Y 배열</param>
        public void Update(double[] yArray)
        {
            Update(null, yArray);
        }

        #endregion

        #region 그리기 - Draw(graphics, reactangle, lowerX, upperX, lowerY, upperY)

        /// <summary>
        /// 그리기
        /// </summary>
        /// <param name="graphics">그래픽스</param>
        /// <param name="reactangle">사각형</param>
        /// <param name="lowerX">하한 X</param>
        /// <param name="upperX">상한 X</param>
        /// <param name="lowerY">하한 Y</param>
        /// <param name="upperY">상한 Y</param>
        public void Draw(Graphics graphics, Rectangle reactangle, double lowerX, double upperX, double lowerY, double upperY)
        {
            double[] xArray = XArray;
            double[] yArray = YArray;

            if(yArray == null)
            {
                return;
            }

            int length = xArray == null ? yArray.Length : yArray.Length > xArray.Length ? xArray.Length : yArray.Length;

            double endX = 0;
            double endY = 0;

            upperX = upperX == lowerX ? lowerX + 1 : upperX;
            upperY = upperY == lowerY ? lowerY + 1 : upperY;

            double xScale = reactangle.Width  / (upperX - lowerX);
            double yScale = reactangle.Height / (upperY - lowerY);

            using(Pen pen = new Pen(LineColor, LineWidth))
            {
                for(int i = 0; i < length; i++)
                {
                    double startX = xArray == null ? i : xArray[i];
                    double startY = yArray[i];

                    startX = reactangle.X + ((startX - lowerX) * xScale);
                    startY = reactangle.Y + reactangle.Height - ((startY - lowerY) * yScale);

                    if(i != 0)
                    {
                        graphics.DrawLine(pen, (float)startX, (float)startY, (float)endX, (float)endY);
                    }

                    endX = startX;
                    endY = startY;
                }
            }
        }

        #endregion
    }
}