﻿using System;
using System.ComponentModel;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 메인 폼
    /// </summary>
    public partial class MainForm : Form
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 사운드 소스
        /// </summary>
        protected SoundSource soundSource = null;

        /// <summary>
        /// 사운드 재생
        /// </summary>
        protected SoundPlayback soundPlayback = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainForm()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainForm()
        {
            InitializeComponent();

            this.listenButton.Click += listenButton_Click;
            this.fileButton.Click   += fileButton_Click;
            this.timer.Tick         += timer_Tick;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region 종료시 처리하기 - OnClosing(e)

        /// <summary>
        /// 종료시 처리하기
        /// </summary>
        /// <param name="e">이벤트 인자</param>
        protected override void OnClosing(CancelEventArgs e)
        {
            if(this.soundSource != null)
            {
                this.soundSource.Dispose(true);
            }

            if(this.soundPlayback != null)
            {
                this.soundPlayback.Dispose(true);
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 사운드 소스 데이터 처리하기 - soundSource_ProcessData(valueArray)

        /// <summary>
        /// 사운드 소스 데이터 처리하기
        /// </summary>
        /// <param name="valueArray">값 배열</param>
        private void soundSource_ProcessData(float[] valueArray)
        {
            this.soundVisualizer.AppendData(valueArray);

            this.soundVisualizer.StartTimer();
        }

        #endregion
        #region 사운드 소스 FFT 데이터 처리하기 - soundSource_ProcessFFTData(valueArray)

        /// <summary>
        /// 사운드 소스 FFT 데이터 처리하기
        /// </summary>
        /// <param name="valueArray">값 배열</param>
        private void soundSource_ProcessFFTData(float[] valueArray)
        {
            float[] temporaryValueArray = new float[valueArray.Length / 2];

            for(int i = 0; i < temporaryValueArray.Length; i++)
            {
                temporaryValueArray[i] = valueArray[i];
            }

            if(this.graphControl.LineList.Count < 1)
            {
                this.graphControl.LineList.Add(new Line());
            }

            this.graphControl.LineList[0].Update(temporaryValueArray);

            this.graphControl.StartTimer();
        }

        #endregion
        #region 실시간 청취 버튼 클릭시 처리하기 - listenButton_Click(sender, e)

        /// <summary>
        /// 실시간 청취 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void listenButton_Click(object sender, EventArgs e)
        {
            if(this.soundSource != null)
            {
                if(this.soundSource.IsCapturing)
                {
                    this.soundSource.Stop();
                }
                else
                {
                    this.soundSource.Start();
                }
            }
            else
            {
                this.soundSource = new SoundSource();

                this.soundSource.ProcessData    += soundSource_ProcessData;
                this.soundSource.ProcessFFTData += soundSource_ProcessFFTData;

                this.soundSource.Start();
            }
        }

        #endregion
        #region 파일 분석 버튼 클릭시 처리하기 - fileButton_Click(sender, e)

        /// <summary>
        /// 파일 분석 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void fileButton_Click(object sender, EventArgs e)
        {
            if(this.soundPlayback == null)
            {
                this.soundPlayback = new SoundPlayback();
            }

            DialogResult result = this.openFileDialog.ShowDialog();

            if(result != DialogResult.OK)
            {
                return;
            }

            this.soundPlayback.Setup(this, this.openFileDialog.FileName);

            this.timer.Enabled = true;
        }

        #endregion
        #region 타이머 틱 처리하기 - timer_Tick(sender, e)

        /// <summary>
        /// 타이머 틱 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void timer_Tick(object sender, EventArgs e)
        {
            float[] valueArray = new float[1024];
 
            if(this.soundPlayback.GetData(valueArray) == false)
            {
                this.timer.Enabled = false;

                return;
            }
     
            soundSource_ProcessData(valueArray);

            float[] fftArray = new float[valueArray.Length];

            FFT.ComputeForwardFFT(valueArray, fftArray);

            soundSource_ProcessFFTData(fftArray);
            soundSource_ProcessData(fftArray);
        }

        #endregion
    }
}