﻿using Microsoft.DirectX.DirectSound;
using System;
using System.IO;
using System.Windows.Forms;

namespace TestProject
{
    /// <summary>
    /// 사운드 재생
    /// </summary>
    public class SoundPlayback : IDisposable
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 장치
        /// </summary>
        protected Device device = null;

        /// <summary>
        /// 2차 버퍼
        /// </summary>
        protected SecondaryBuffer secondaryBuffer = null;

        /// <summary>
        /// 위치
        /// </summary>
        protected int location = 0;

        /// <summary>
        /// 리소스 해제 여부
        /// </summary>
        protected bool disposed = false;

        #endregion


        //////////////////////////////////////////////////////////////////////////////////////////////////// Destructor

        #region 소멸자 - ~SoundPlayback()

        /// <summary>
        /// 소멸자
        /// </summary>
        ~SoundPlayback()
        {
            Dispose(false);
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 셋업하기 - Setup(ownerControl, filePath)

        /// <summary>
        /// 셋업하기
        /// </summary>
        /// <param name="ownerControl">소유자 컨트롤</param>
        /// <param name="filePath">파일 경로</param>
        public void Setup(Control ownerControl, string filePath)
        {
            this.device = new Device();

            this.device.SetCooperativeLevel(ownerControl, CooperativeLevel.Priority);

            this.secondaryBuffer = new SecondaryBuffer(filePath, this.device);

            this.secondaryBuffer.SetCurrentPosition(0);
        }

        #endregion
        #region 데이터 구하기 - GetData(valueArray)

        /// <summary>
        /// 데이터 구하기
        /// </summary>
        /// <param name="valueArray">값 배열</param>
        public bool GetData(float[] valueArray)
        {
            try
            {
                int bitCountPerSample = this.secondaryBuffer.Format.BitsPerSample;

                byte[] byteArray = new byte[valueArray.Length * bitCountPerSample / 8];

                MemoryStream stream = new MemoryStream(byteArray);

                stream.Position = 0;

                this.secondaryBuffer.Read(this.location, stream, valueArray.Length, LockFlag.None);

                this.location += valueArray.Length;

                int k = 0;

                if(bitCountPerSample == 16)
                {
                    for(int i = 0; i < valueArray.Length; i++)
                    {
                        int lower = byteArray[k++];
                        int upper = byteArray[k++];

                        valueArray[i] = (float)(lower + (upper * 256));
                    }
                }
                else
                {
                    for(int i = 0; i < valueArray.Length; i++)
                    {
                        valueArray[i] = (float)(byteArray[k++]);
                    }
                }
            }
            catch(Exception)
            {
                return false;
            }

            return true;
        }

        #endregion

        #region 리소스 해제하기 - Dispose(disposing)

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        /// <param name="disposing">리소스 해제 여부</param>
        public void Dispose(bool disposing)
        {
            if(this.disposed)
            {
                return;
            }

            this.disposed = true;
            
            GC.SuppressFinalize(this);

            this.secondaryBuffer.Dispose();

            this.device.Dispose();
        }

        #endregion
        #region 리소스 해제하기 - IDisposable.Dispose()

        /// <summary>
        /// 리소스 해제하기
        /// </summary>
        void IDisposable.Dispose()
        {
            Dispose(true);
        }

        #endregion
    }
}