﻿using System;
using System.Drawing;

namespace TestProject
{
    /// <summary>
    /// 스택
    /// </summary>
    public class Stack
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Static
        //////////////////////////////////////////////////////////////////////////////// Protected

        #region Field

        /// <summary>
        /// 색상 배열
        /// </summary>
        protected static Color[] _colorArray;

        /// <summary>
        /// 감마 수정 배열
        /// </summary>
        protected static int[] _gammaCorrectionArray = new int[1000];

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region Field

        /// <summary>
        /// 색상 배열
        /// </summary>
        public Color[] ColorArray = null;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Static

        #region 생성자 - Stack()

        /// <summary>
        /// 생성자
        /// </summary>
        static Stack()
        {
            Color[] colorArray =
            {
                Color.Black,
                Color.Blue,
                Color.Green,
                Color.GreenYellow,
                Color.Yellow,
                Color.Orange,
                Color.OrangeRed,
                Color.Red
            };

            _colorArray = new Color[colorArray.Length * 100];

            int k = 0;

            for(int i = 0; i < colorArray.Length - 1; i++)
            {
                for(int j = 0; j < 100; j++)
                {
                    Color lowColor  = colorArray[i];
                    Color highColor = colorArray[i + 1];

                    int r = ((lowColor.R * (100 - j)) + (highColor.R * j)) / 100;
                    int g = ((lowColor.G * (100 - j)) + (highColor.G * j)) / 100;
                    int b = ((lowColor.B * (100 - j)) + (highColor.B * j)) / 100;

                    _colorArray[k++] = Color.FromArgb(r, g, b);
                }
            }

            float gammaCorrection = 0.3f;

            float gain = (float)((_colorArray.Length - 1) / Math.Pow(_gammaCorrectionArray.Length, gammaCorrection));

            for(int i = 0; i < _gammaCorrectionArray.Length; i++)
            {
                int value = (int)(gain * Math.Pow((double)i, gammaCorrection));

                value = value < 0 ? 0 : value > _colorArray.Length - 1 ? _colorArray.Length - 1 : value;

                _gammaCorrectionArray[i] = value;
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Instance
        //////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - Stack(valueArray, minimumValue, maximumValue)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="valueArray">값 배열</param>
        /// <param name="maximumValue">최소 값</param>
        /// <param name="minimumValue">최대 값</param>
        public Stack(float[] valueArray, double minimumValue, double maximumValue)
        {
            float scale = (float)(_gammaCorrectionArray.Length / (maximumValue - minimumValue));

            ColorArray = new Color[valueArray.Length / 2];

            for(int i = 0; i < ColorArray.Length; i++)
            {
                int value = (int)((valueArray[i] - minimumValue) * scale);

                value = value < 0 ? 0 : value > _colorArray.Length - 1 ? _colorArray.Length - 1 : value;
                value = _gammaCorrectionArray[value];

                ColorArray[i] = _colorArray[value];
            }
        }

        #endregion
    }
}